/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <string.h>
#include <SDL/SDL_types.h>

#include "pia.h"
#include "logging.h"

pia_port PIA_0A, PIA_0B, PIA_1A, PIA_1B;
extern int trace;

void pia_init(void) {
	memset(&PIA_0A, 0, sizeof(pia_port));
	memset(&PIA_0B, 0, sizeof(pia_port));
	memset(&PIA_1A, 0, sizeof(pia_port));
	memset(&PIA_1B, 0, sizeof(pia_port));
}

void pia_control_write(pia_port *p, Uint8 byte) {
	p->interrupt_enable = byte & 0x01;
	p->interrupt_transition = byte & 0x02;
	p->register_select = byte & 0x04;
	p->control_register = byte;
}

Uint8 pia_control_read(pia_port *p) {
	return p->control_register;
}

/* A 1 in DDR bits means output */

/* This is for MPU writes to the PIA */
void pia_data_write(pia_port *p, Uint8 byte) {
	int i;
	if (!p->register_select) {
		p->direction_register = byte;
		return;
	}
	byte &= p->direction_register;
	p->output_register &= ~p->direction_register;
	p->output_register |= byte;
	for (i = 0; i < MAX_WRFUNCS; i++) {
		if (p->wrfunc[i]) {
			p->wrfunc[i](p->output_register);
		}
	}
}

Uint8 pia_data_read(pia_port *p) {
	p->control_register &= 0x3f;
	return p->output_register;
}

void pia_set_ca1(pia_port *p) {
	if (p->interrupt_enable) {
		p->control_register |= 0x80;
		if (p->intfunc) {
			p->intfunc();
		}
	}
}

/* This is called by devices to set input bits */
void pia_data_set(pia_port *p, Uint8 byte) {
	byte &= ~(p->direction_register);
	p->output_register &= p->direction_register;
	p->output_register |= byte;
}
