/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <SDL/SDL_types.h>

#include "m6809.h"
#include "sam.h"
#include "snapshot.h"
#include "logging.h"

/* Pretty lousy snapshot code - just to assist in testing.  Doesn't
 * store SAM or PIA states. */

int snapshot_state = 0;

int write_snapshot(char *filename) {
	Uint8 regdump[14];
	int fd;
	fd = open(filename, O_CREAT|O_WRONLY);
	if (fd == -1) {
		LOG_WARN2("Failed to write snapshot '%s': %s.\n", filename,
				strerror(errno));
		return 0;
	}
	regdump[0] = reg_cc;
	regdump[1] = reg_accum.reg_ab.reg_a;
	regdump[2] = reg_accum.reg_ab.reg_b;
	regdump[3] = reg_dp;
	regdump[4] = reg_x >> 8;
	regdump[5] = reg_x & 0xff;
	regdump[6] = reg_y >> 8;
	regdump[7] = reg_y & 0xff;
	regdump[8] = reg_u >> 8;
	regdump[9] = reg_u & 0xff;
	regdump[10] = reg_s >> 8;
	regdump[11] = reg_s & 0xff;
	regdump[12] = reg_pc >> 8;
	regdump[13] = reg_pc & 0xff;
	write(fd, regdump, sizeof(regdump));
	write(fd, ram0, sizeof(ram0));
	close(fd);
	return 1;
}

int read_snapshot(char *filename) {
	Uint8 regdump[14];
	int fd;
	fd = open(filename, O_RDONLY);
	if (fd == -1) {
		LOG_WARN2("Failed to read snapshot '%s': %s.\n", filename,
				strerror(errno));
		return 0;
	}
	read(fd, regdump, sizeof(regdump));
	reg_cc = regdump[0];
	reg_accum.reg_ab.reg_a = regdump[1];
	reg_accum.reg_ab.reg_b = regdump[2];
	reg_dp = regdump[3];
	reg_x = regdump[4] << 8;
	reg_x |= regdump[5];
	reg_y = regdump[6] << 8;
	reg_y |= regdump[7];
	reg_u = regdump[8] << 8;
	reg_u |= regdump[9];
	reg_s = regdump[10] << 8;
	reg_s |= regdump[11];
	reg_pc = regdump[12] << 8;
	reg_pc |= regdump[13];
	read(fd, ram0, sizeof(ram0));
	close(fd);
	return 1;
}
