/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>

#include "pia.h"
#include "sound.h"
#include "logging.h"

/* Seriously crufty sound code - basically the main loop calls sound_cycle
 * often enough that the buffer is just about full by the time the SDL audio
 * system wants it. */

SDL_AudioSpec *audio;

Uint8 sound_buffer[SOUND_BUFFER_SIZE];
Uint8 *sound_next = sound_buffer;
int sound_len = 0;

static void sound_callback(void *userdata, Uint8 *stream, int len);

void sound_init(void) {
	memset(sound_buffer, 0, sizeof(sound_buffer));
	audio = malloc(sizeof(SDL_AudioSpec));
	audio->freq = 44100;
	audio->format = AUDIO_S8;
	audio->channels = 1;
	audio->samples = SOUND_BUFFER_SIZE;
	audio->callback = sound_callback;
	audio->userdata = NULL;
	if (SDL_OpenAudio(audio, NULL) < 0) {
		LOG_ERROR1("Couldn't open audio: %s\n", SDL_GetError());
		exit(1);
	}
	SDL_PauseAudio(0);
}

void sound_cycle(void) {
	if (sound_len < SOUND_BUFFER_SIZE) {
		*(sound_next++) = (PIA_1B.control_register & 8) ?
			(PIA_1A.output_register & 0xfc) : audio->silence;
		sound_len++;
	}
}

static void sound_callback(void *userdata, Uint8 *stream, int len) {
	if (sound_len == 0)
		return;
	if (len > sound_len) {
		len = sound_len;
		memset(stream + sound_len, audio->silence, len - sound_len);
	}
	memcpy(stream, sound_buffer, len);
	sound_next = sound_buffer;
	sound_len = 0;
}
